@@ -25,6 +25,8 @@ import ConfigParser
 from txclib import utils, project
 from txclib.utils import parse_json, compile_json, relpath
 from txclib.config import OrderedRawConfigParser
+from txclib.exceptions import UnInitializedError
+
 
 def cmd_init(argv, path_to_tx):
     "Initialize a new transifex project."
@@ -79,6 +81,8 @@ def cmd_init(argv, path_to_tx):
 
     if not transifex_host:
         transifex_host = default_transifex
+    if not transifex_host.startswith(('http://', 'https://')):
+        transifex_host = 'https://' + transifex_host
 
     config_file = os.path.join(path_to_tx, ".tx", "config")
     if not os.path.exists(config_file):
@@ -100,6 +104,7 @@ def cmd_init(argv, path_to_tx):
 
     utils.MSG("Done.")
 
+
 def cmd_set(argv, path_to_tx):
     "Add local or remote files under transifex"
 
@@ -218,8 +223,12 @@ def cmd_set(argv, path_to_tx):
 
         # Calculate relative path
         path_to_file = relpath(args[0], path_to_tx)
-        # Chdir to the root dir
-        os.chdir(path_to_tx)
+
+        try:
+            _go_to_dir(path_to_tx)
+        except UnInitializedError, e:
+            utils.ERRMSG(e)
+            return
 
         if not utils.valid_slug(resource):
             parser.error("Invalid resource slug. The format is <project_slug>"\
@@ -231,12 +240,12 @@ def cmd_set(argv, path_to_tx):
 
     return
 
+
 def _auto_local(path_to_tx, resource, source_language, expression, execute=False, source_file=None,
     nosource=False, regex=False):
     """
     Auto configure local project
     """
-
     # The path everything will be relative to
     curpath = os.path.abspath(os.curdir)
 
@@ -308,6 +317,7 @@ def _auto_local(path_to_tx, resource, source_language, expression, execute=False
 
     prj.save()
 
+
 def _auto_remote(path_to_tx, url):
     """
     Initialize a remote release/project/resource to the current directory.
@@ -332,10 +342,10 @@ def _auto_remote(path_to_tx, url):
             project = vars['project'], release = vars['release'])
         resources = []
         for r in rel_info['resources']:
-            if r.has_key('project_slug'):
-                resources.append(r['project_slug'])
+            if r.has_key('project'):
+                resources.append('.'.join([r['project']['slug'], r['slug']]))
             else:
-                resources.append('.'.join(vars['project'], r['slug']))
+                resources.append('.'.join([vars['project'], r['slug']]))
         utils.MSG("%s resources found. Configuring..." % len(resources))
     elif type == 'resource':
         utils.MSG("Getting details for resource %s" % vars['resource'])
@@ -350,7 +360,7 @@ def _auto_remote(path_to_tx, url):
              username, password, hostname = vars['hostname'],
              project = proj, resource=res)
         try:
-            source_lang = res_info['source_language']['code']
+            source_lang = res_info['source_language_code']
             i18n_type = res_info['i18n_type']
         except KeyError:
             raise Exception("Remote server seems to be running an unsupported version"
@@ -364,6 +374,7 @@ def _auto_remote(path_to_tx, url):
 
     prj.save()
 
+
 def cmd_push(argv, path_to_tx):
     "Push local files to remote server"
     usage="usage: %prog [tx_options] push [options]"
@@ -414,22 +425,20 @@ def cmd_push(argv, path_to_tx):
 
     # instantiate the project.Project
     prj = project.Project(path_to_tx)
-    available_resources = prj.get_resource_list()
-    for r in resources:
-        if not r in available_resources:
-            raise Exception("Specified resource '%s' does not exist." % r)
-
     if not (options.push_source or options.push_translations):
-        parser.error("You need to specify at least on of the -s|--source,"
+        parser.error("You need to specify at least one of the -s|--source,"
             " -t|--translations flags with the push command.")
 
-    prj.push(force=force_creation, resources=resources, languages=languages,
+    prj.push(
+        force=force_creation, resources=resources, languages=languages,
         skip=skip, source=options.push_source,
         translations=options.push_translations,
-        no_interactive=options.no_interactive)
+        no_interactive=options.no_interactive
+    )
 
     utils.MSG("Done.")
 
+
 def cmd_pull(argv, path_to_tx):
     "Pull files from remote server to local repository"
     usage="usage: %prog [tx_options] pull [options]"
@@ -463,6 +472,10 @@ def cmd_pull(argv, path_to_tx):
         help="By default transifex will fetch new translations files and"\
             " replace existing ones. Use this flag if you want to disable"\
             " this feature")
+    parser.add_option("--minimum-perc", action="store", type="int",
+        dest="minimum_perc", default=0,
+        help="Specify the minimum acceptable percentage of a translation "
+             "in order to download it.")
 
     (options, args) = parser.parse_args(argv)
 
@@ -481,21 +494,25 @@ def cmd_pull(argv, path_to_tx):
         resources = []
 
     skip = options.skip_errors
+    minimum_perc = options.minimum_perc or None
+
+    try:
+        _go_to_dir(path_to_tx)
+    except UnInitializedError, e:
+        utils.ERRMSG(e)
+        return
 
-    os.chdir(path_to_tx)
     # instantiate the project.Project
     prj = project.Project(path_to_tx)
-    available_resources = prj.get_resource_list()
-    for r in resources:
-        if not r in available_resources:
-            raise Exception("Specified resource '%s' does not exist." % r)
-
-    prj.pull(languages=languages, resources=resources, overwrite=options.overwrite,
+    prj.pull(
+        languages=languages, resources=resources, overwrite=options.overwrite,
         fetchall=options.fetchall, fetchsource=options.fetchsource,
-        force=options.force, skip=skip)
+        force=options.force, skip=skip, minimum_perc=minimum_perc
+    )
 
     utils.MSG("Done.")
 
+
 def _set_source_file(path_to_tx, resource, lang, path_to_file):
     """Reusable method to set source file."""
 
@@ -508,8 +525,11 @@ def _set_source_file(path_to_tx, resource, lang, path_to_file):
     if not lang:
         raise Exception("You haven't specified a source language.")
 
-    # Chdir to the root dir
-    os.chdir(path_to_tx)
+    try:
+        _go_to_dir(path_to_tx)
+    except UnInitializedError, e:
+        utils.ERRMSG(e)
+        return
 
     if not os.path.exists(path_to_file):
         raise Exception("tx: File ( %s ) does not exist." %
@@ -555,8 +575,11 @@ def _set_translation(path_to_tx, resource, lang, path_to_file):
             " be in the following format project_slug.resource_slug." %
             resource)
 
-    # Chdir to the root dir
-    os.chdir(path_to_tx)
+    try:
+        _go_to_dir(path_to_tx)
+    except UnInitializedError, e:
+        utils.ERRMSG(e)
+        return
 
     # Warn the user if the file doesn't exist
     if not os.path.exists(path_to_file):
@@ -581,6 +604,7 @@ def _set_translation(path_to_tx, resource, lang, path_to_file):
 
     prj.save()
 
+
 def cmd_status(argv, path_to_tx):
     "Print status of current project"
 
@@ -598,12 +622,7 @@ def cmd_status(argv, path_to_tx):
         resources = []
 
     prj = project.Project(path_to_tx)
-    available_resources = prj.get_resource_list()
-    for r in resources:
-        if not r in available_resources:
-            raise Exception("Specified resource '%s' does not exist." % r)
-    if not resources:
-        resources = available_resources
+    resources = prj.get_chosen_resources(resources)
     resources_num = len(resources)
     for id, res in enumerate(resources):
         p, r = res.split('.')
@@ -626,6 +645,7 @@ def cmd_status(argv, path_to_tx):
 
         utils.MSG("")
 
+
 def cmd_help(argv, path_to_tx):
     "List all available commands"
 
@@ -664,3 +684,69 @@ def cmd_help(argv, path_to_tx):
         utils.MSG("  %-15s\t%s" % (key, fns[key].func_doc))
 
     utils.MSG("\nFor more information run %s command --help" % sys.argv[0])
+
+
+def cmd_delete(argv, path_to_tx):
+    "Delete an accessible resource or translation in a remote server."
+
+    class EpilogParser(OptionParser):
+       def format_epilog(self, formatter):
+           return self.epilog
+
+    usage="usage: %prog [tx_options] delete OPTION [OPTIONS]"
+    description=(
+        "This command deletes translations for a resource in the remote server."
+    )
+    epilog="\nExamples:\n"\
+        " To delete a translation:\n  "\
+        "$ tx delete -r project.resource -l <lang_code>\n\n"\
+        " To delete a resource:\n  $ tx delete -r project.resource\n"
+    parser = EpilogParser(usage=usage, description=description, epilog=epilog)
+    parser.add_option(
+        "-r", "--resource", action="store", dest="resources", default=None,
+        help="Specify the resource you want to delete (defaults to all)"
+    )
+    parser.add_option("-l","--language", action="store", dest="languages",
+        default=None, help="Specify the translation you want to delete")
+    parser.add_option(
+        "--skip", action="store_true", dest="skip_errors", default=False,
+        help="Don't stop on errors."
+    )
+    parser.add_option("-f","--force", action="store_true", dest="force_delete",
+        default=False, help="Delete an entity forcefully.")
+
+    (options, args) = parser.parse_args(argv)
+
+    if options.languages:
+        languages = options.languages.split(',')
+    else:
+        languages = []
+
+    if options.resources:
+        resources = options.resources.split(',')
+    else:
+        resources = []
+
+    skip = options.skip_errors
+    force = options.force_delete
+
+    prj = project.Project(path_to_tx)
+    prj.delete(resources, languages, skip, force)
+    utils.MSG("Done.")
+
+
+def _go_to_dir(path):
+    """Change the current working directory to the directory specified as
+    argument.
+
+    Args:
+        path: The path to chdor to.
+    Raises:
+        UnInitializedError, in case the directory has not been initialized.
+    """
+    if path is None:
+        raise UnInitializedError(
+            "Directory has not been initialzied. "
+            "Did you forget to run 'tx init' first?"
+        )
+    os.chdir(path)
