@@ -6,31 +6,22 @@ except ImportError:
 import urllib2 # This should go and instead use do_url_request everywhere
 
 from urls import API_URLS
+from txclib.log import logger
 
-# This is a mapping between i18n types supported on Transifex and related file
-# extensions of the translation files.
-FILE_EXTENSIONS = {
-    'PO': 'po',
-    'QT': 'ts',
-    'INI': 'ini',
-    'PROPERTIES': 'properties',
-    'ANDROID': 'xml',
-    'XHTML': 'xhtml',
-    'HTML': 'html',
-}
 
 def MSG(msg, verbose=1):
     """
     STDOUT logging function
     """
-    if verbose:
-        sys.stdout.write('%s\n' % msg)
+    logger.info('%s' % msg)
+
 
 def ERRMSG(msg, verbosity=1):
     """
     STDERR logging function
     """
-    sys.stderr.write('%s\n' % msg)
+    logger.error('%s' % msg)
+
 
 def find_dot_tx(path = os.path.curdir, previous = None):
     """
@@ -66,11 +57,12 @@ def regex_from_filefilter(file_filter, root_path = os.path.curdir):
 
 
 TX_URLS = {
-    'resource': '(?P<hostname>https?://(\w|\.|:)+)/projects/p/(?P<project>(\w|-)+)/resource/(?P<resource>(\w|-)+)/?$',
-    'release': '(?P<hostname>https?://(\w|\.|:)+)/projects/p/(?P<project>(\w|-)+)/r/(?P<release>(\w|-)+)/?$',
-    'project': '(?P<hostname>https?://(\w|\.|:)+)/projects/p/(?P<project>(\w|-)+)/?$',
+    'resource': '(?P<hostname>https?://(\w|\.|:|-)+)/projects/p/(?P<project>(\w|-)+)/resource/(?P<resource>(\w|-)+)/?$',
+    'release': '(?P<hostname>https?://(\w|\.|:|-)+)/projects/p/(?P<project>(\w|-)+)/r/(?P<release>(\w|-)+)/?$',
+    'project': '(?P<hostname>https?://(\w|\.|:|-)+)/projects/p/(?P<project>(\w|-)+)/?$',
 }
 
+
 def parse_tx_url(url):
     """
     Try to match given url to any of the valid url patterns specified in
@@ -84,6 +76,7 @@ def parse_tx_url(url):
 
     raise Exception("tx: Malformed url given!")
 
+
 def get_details(api_call, username, password, *args, **kwargs):
     """
     Get the tx project info through the API.
@@ -115,6 +108,7 @@ def get_details(api_call, username, password, *args, **kwargs):
 
     return remote_project
 
+
 def valid_slug(slug):
     """
     Check if a slug contains only valid characters.
@@ -149,9 +143,11 @@ def discover_commands():
 
     return command_table
 
+
 class UnknownCommandError(Exception):
 	pass
 
+
 def exec_command(command, *args, **kwargs):
     """
     Execute given command
@@ -163,6 +159,7 @@ def exec_command(command, *args, **kwargs):
         raise UnknownCommandError
     cmd_fn(*args,**kwargs)
 
+
 def mkdir_p(path):
     try:
         if path:
@@ -184,6 +181,7 @@ COLORS = [
 
 DISABLE_COLORS = False
 
+
 def color_text(text, color_name, bold=False):
     """
     This command can be used to colorify command line output. If the shell
